<?php
/* 
	Fonctions stagiaires
	Circon'script PHP Script
	Version 7.0.3
	CeCILL Copyright (C) 2008 Thierry Gangloff
	e-mail: thierry.gangloff@ac-strasbourg.fr
*/
/*NormaliseNom
	Met une majuscule aux premires lettres d'un nom, mme compos.
*/
function majuscule($nom)
{
	$nom=strtolower($nom);
	$parties=explode(" ",$nom);
	$nom_nouv='';
	foreach ($parties as $p) {
		$p1=ucfirst(trim($p));
		if ($p1) {
			if ($nom_nouv) $nom_nouv.=' ';
			$nom_nouv.=$p1;
		}
	}	
	$parties=explode("-",$nom_nouv);
	$nom_nouv='';
	foreach ($parties as $p) {
		$p1=ucfirst(trim($p));
		if ($p1) {
			if ($nom_nouv) $nom_nouv.='-';
			$nom_nouv.=$p1;
		}
	}	
	$parties=explode("'",$nom_nouv);
	$nom_nouv='';
	foreach ($parties as $p) {
		$p1=ucfirst(trim($p));
		if ($p1) {
			if ($nom_nouv) $nom_nouv.="'";
			$nom_nouv.=$p1;
		}
	}	
	return $nom_nouv;
}
/*
	Retourne vrai si un nom est correct.
	Faux s'il contient des caractre errons.
	On accepte les chiffres dans les noms,
*/
function nomValide($nom)
{
	return preg_match("#^[abcdefghijklmnopqrstuvwxyz 0123456789'\-]+$#",strtolower($nom));
}
/*
	Retourne le nom entirement en majuscules, dbarass de tous ses accents
	! le nom ne doit contenir que des caractres lgaux
	Les traits d'union sont remplacs par un espace
	Idem pour l'apostrophe
	Utilis pour trouver un nom/prnom dans les champs nom_f et prenom_f
*/

function normaliseNom($nom)
{
	$nom=strtoupper($nom);
	$accents=strtoupper("-'");
	$rempl="AAAAAACEEEEIIIIONOOOOOUUUUYY  ";
	return trim(strtr($nom,$accents,$rempl));
}



/*
	Regles: retrouve les rgles de gestion des quota dans un tableau associatif:
		quota => quota  attribuer
		id_ecole => id de l'cole (0 = pas de rgle concernant l'cole)
		nivs => tableau des niveaux ('/CP/', '/CE1/',..... , '') (contient '//' si une rgle concerne un niveau indfini)
		id_groupe => id du groupe du stagiaire
*/
function regles($id_circo,$h)
{
	$regles=array();
	$r=query("SELECT quota,id_ecole,niveau,id_groupe FROM regles WHERE id_circo=$id_circo",$h);
	while ($row=@mysqli_fetch_array($r)) {
		$niveau=$row['niveau'];
		$nivs=array();
		if ($niveau) {
			if ($niveau=='//') $nivs[]='//';
			else {
				$t=explode('/',$niveau);
				if (!empty($t)) foreach($t as $v) if ($v) $nivs[]='/'.$v.'/';
			}
		}
		$quota=$row['quota'];
		$id_ecole=$row['id_ecole'];
		$id_groupe=$row['id_groupe'];
		$regles[]=array('quota'=>$quota,'id_ecole'=>$id_ecole,'niveaux'=>$nivs, 'id_groupe'=>$id_groupe);
	}
	return $regles;
}
/*
	retourne le quota d'heures de rfrences  faire pour un stagaire
	Si aucune rgle ne s'applique, retourne -1
*/
function quotaReferenceStagiaire($id_ecole,$id_groupe,$niveau, $regles)
{
	$quota=-1;
	foreach($regles as $regle) {

		if($regle['id_ecole']==0  && $regle['id_groupe']==0 && count($regle['niveaux'])==0) continue; //limine trivial possible
		if($regle['id_ecole']>0) 
			if( $id_ecole!=$regle['id_ecole']) continue;
		if ($regle['id_groupe']>0) 
			if($id_groupe!=$regle['id_groupe']) continue;
		if (count($regle['niveaux'])>0) {
			$trouve=false;
			foreach($regle['niveaux'] as $niv) {
				if ($niv=='//' && $niveau=='') {
					$trouve=true;
					break;
				}
				else if ($niv!='//' && $niveau!='' && strpos($niveau,$niv)!==false) {
					$trouve=true;
					break;
				}
			}
			if (!$trouve) continue;
		}
		// tous les tests russis. 
		$quota=$regle['quota'];
	}
	return $quota;
}

/*
	Calcule le quota rel  faire par le stagaire
	aprs application de la rgle d'arrondi.
	SANS calcul des rgles particulires de gestion.
*/
function quotaReelStagiaire($quota_referent,$quotite,$duree_seance,$regle)
{
	$quota_reel=$quota_referent*$quotite/100.0;
	if ($regle==0) return $quota_reel;
	switch($regle) {
		case 0: 
			return $quota_reel; // rgle de trois 
		case 1: //arrondi au plus proche, si quidistant vers le bas.
			if ($duree_seance==0) return 0.0;
			$nb_seances_reel=$quota_reel/$duree_seance;
			$nb_seances=intval($nb_seances_reel);
			if ($nb_seances_reel-$nb_seances>0.5) ++$nb_seances;
			return $nb_seances*$duree_seance;
		case 2: // arrondi au plus proche, si quidistant vers le haut
			if ($duree_seance==0) return 0.0;
			$nb_seances_reel=$quota_reel/$duree_seance;
			$nb_seances=intval($nb_seances_reel);
			if ($nb_seances_reel-$nb_seances>=0.5) ++$nb_seances;
			return $nb_seances*$duree_seance;
		case 3: // arrondi infrieur
			if ($duree_seance==0) return 0.0;
			$nb_seances_reel=$quota_reel/$duree_seance;
			$nb_seances=intval($nb_seances_reel);
			return $nb_seances*$duree_seance;
		case 4: //arrondi suprieur
			if ($duree_seance==0) return 0.0;
			$nb_seances_reel=$quota_reel/$duree_seance;
			$nb_seances=intval($nb_seances_reel);
			if ($nb_seances_reel-$nb_seances>0.01) ++$nb_seances;
			return $nb_seances*$duree_seance;
	}
	return 0.0; 		
}
/* 
	Convocation automatique d'un stagiaire a une liste de sances
	Optimis pour viter de trop frquentes lectures de tables
*/
function convoquerAuto($id_stagiaire, $id_liste,$id_circo,$h)
{
	global $erreur;
	$r=query("SELECT id_groupe,quota_reel FROM stagiaires  WHERE id_stagiaire=$id_stagiaire",$h);
	$row=@mysqli_fetch_row($r);
	$id_groupe=$row[0];
	$quota_reel=$row[1];	
	$nb=0;
	$tri_prec=0;
	$duree_totale_stag=-1;
	$dates_stag_lues=false;
	$dates_stag=array();
	$horaires_stag=array();
	$durees_stag=array();
	// si le stag. est dans un groupe mais que ce groupe ne contient aucune sance, alors c'est comme s'il n'y avait pas de groupes
	if ($id_groupe) {
		$n=queryValue("SELECT COUNT(*) FROM groupements WHERE id_groupe=$id_groupe",$h);
		if (!$n) $id_groupe=0;
	}	// on extrait la liste dees sances pour lesquelles le stagiaire n'est pas dja convoqu, qui appartiennent a la liste et qui appartiennent au groupe du stagiaire si ce dernier en possede.
	//par prcaution,on n'extrait que les sances rellement partages (en principe, il ne doit pas y en avoir d'autres dans la base, mais sait-on jamais...)
	// on n'extait que les sances pour lesquelles le stagiaire n'est pas dja convoqu.
	if ($id_groupe==0)
		$r=query("SELECT items.id_seance,items.tri FROM (items, partages,seances,animations,circos) LEFT JOIN convocations ON convocations.id_stagiaire=$id_stagiaire AND convocations.id_seance=items.id_seance  WHERE items.id_liste=$id_liste AND items.id_seance=partages.id_seance AND partages.id_circo=$id_circo AND partages.accepte<>0 AND items.id_seance=seances.id_seance AND seances.id_anim=animations.id_anim AND animations.id_circo=circos.id_circo GROUP BY items.id_seance HAVING COUNT(convocations.id_convoc)=0 ORDER BY items.tri ASC, seances.tri ASC",$h);
	else 
		$r=query("SELECT items.id_seance,items.tri FROM (items, partages, seances, animations,circos, groupements) LEFT JOIN convocations ON convocations.id_stagiaire=$id_stagiaire AND convocations.id_seance=items.id_seance  WHERE items.id_liste=$id_liste AND items.id_seance=partages.id_seance AND partages.id_circo=$id_circo AND partages.accepte<>0 AND items.id_seance=seances.id_seance AND seances.id_anim=animations.id_anim AND animations.id_circo=circos.id_circo AND items.id_seance=groupements.id_seance AND groupements.id_groupe=$id_groupe GROUP BY items.id_seance HAVING COUNT(convocations.id_convoc)=0 ORDER BY items.tri ASC, seances.tri ASC",$h);

	if ($erreur) return 0;
	while ($row=@mysqli_fetch_row($r)) {
		if ($erreur) break;
		$id_seance=$row[0];
		$tri=$row[1];
		$autoriser_convoc_plafond=autoriserConvocPlafond($id_seance,$h);
		if($tri>0 && $tri==$tri_prec) continue; // passer les sances ayant le meme tri
		
		// lire les parametres de la sance
		$r2=query("SELECT max,SUM(dates.duree) FROM (seances,dates) WHERE seances.id_seance=$id_seance AND dates.id_seance=$id_seance",$h);
		$row=@mysqli_fetch_row($r2);
		$max=$row[0];
		$duree_seance=$row[1];
		if ($max>0) {
			$nb_conv=queryValue("SELECT COUNT(*) FROM convocations WHERE id_seance=$id_seance",$h);
			if($nb_conv>=$max) continue; // si le maximum est atteint, passer
			if (!$autoriser_convoc_plafond) { // il y a un maximum et l'organisateur n'autorise pas les convocs aux animations plafonnes si pas d'inscription: vrifie s'il existe une inscription
				$n=queryValue("SELECT COUNT(*) FROM inscriptions WHERE id_stagiaire=$id_stagiaire AND id_seance=$id_seance",$h);
				if (!$n) continue;
			}
		}
		if ($tri==0) { //sance obligatoire: convoquer sans tester la dure totale ou le chevauchement de dates
			query("INSERT INTO convocations(id_stagiaire, id_seance) VALUES($id_stagiaire,$id_seance)",$h); 
			++$nb;
		}
		else { //sance facultative
			// passer si le stagiaire est dja convoqu a une autre sance ayant le meme tri
			$n=queryValue("SELECT COUNT(*) FROM items, convocations WHERE items.id_liste=$id_liste AND items.tri=$tri AND items.id_seance=convocations.id_seance AND convocations.id_stagiaire=$id_stagiaire AND convocations.id_seance<>$id_seance",$h);
			if ($n) {
				$tri_prec=$tri; // ne plus continuer avec les sances de ce tri
				continue;
			}
			//calcul du temps restant disponible
			// la dure totale du stagiaire n'est dtermine qu'une fois dans la base. Ensuite, on conserve la dure et on la met a jour a chaque nouvelle convoc.
			if ($duree_totale_stag<0) $duree_totale_stag=queryValue("SELECT SUM(dates.duree) FROM dates, seances,convocations WHERE convocations.id_stagiaire=$id_stagiaire AND convocations.id_seance=seances.id_seance AND seances.id_seance=dates.id_seance",$h);
			if ($duree_totale_stag>=$quota_reel) break; // le stagiaire a atteint son quota horaire: quitter
			if ($duree_totale_stag+$duree_seance > $quota_reel) continue; // passer si la dure de la sance est trop longue
			// vrification de dates - ne pas choisir des sances qui se chevauchent
			$r2=query("SELECT dates.date, dates.horaire,dates.duree FROM dates WHERE id_seance=$id_seance",$h);
			$dates=array(); // 
			$horaires=array();
			$durees=array();
			// cherche toutes les dates de la sance en cours
			while ($row2=@mysqli_fetch_row($r2)) {
				if ($row2[0]!='0000-00-00' && $row2[0]!='9999-12-31' && $row2[0]!='') {  // laisser de cote les dates a definir
					$dates[]=$row2[0];
					$horaires[]=$row2[1];
					$durees[]=$row2[2];	
				}
			}	
			// lire toutes les dates du stagiaire si cela n'a pas dj t fait
			// compare avec les dates du stagiaire
			$ok=true;
			if (!empty($dates)) {	
				if (!$dates_stag_lues) {
					$r2=query("SELECT dates.date,dates.horaire,dates.duree FROM convocations,seances,dates WHERE convocations.id_stagiaire=$id_stagiaire AND seances.id_seance=convocations.id_seance AND dates.id_seance=seances.id_seance",$h);
					while ($row2=@mysqli_fetch_row($r2)) {
							if ($row2[0]!='0000-00-00' && $row2[0]!='9999-12-31' && $row2[0]!='') {
								$dates_stag[]=$row2[0];
								$horaires_stag[]=$row2[1];
								$durees_stag[]=$row2[2];
							}
					}
					$dates_stag_lues=true;
				}
				for ($j=0;$j<count($dates_stag)&&$ok;++$j) 
					for ($i=0;$i<count($dates)&&$ok;++$i) if (intersectionHoraires($dates[$i],$horaires[$i],$durees[$i],$dates_stag[$j],$horaires_stag[$j],$durees_stag[$j])) {
						$ok=false;
						break;
					}		
			}
			if($ok) {
				query("INSERT INTO convocations(id_stagiaire, id_seance) VALUES($id_stagiaire,$id_seance)",$h); 
				$duree_totale_stag+=$duree_seance; 
				// ajoute les dates de la sance en cours aux dates du stagiaire
				if ($dates_stag_lues) 
					for ($i=0;$i<count($dates);++$i) {
						$dates_stag[]=$dates[$i];
						$horaires_stag[]=$horaires[$i];
						$durees_stag[]=$durees[$i];
					}
				$tri_prec=$tri; // et ne plus continuer avec ce tri
				++$nb;
			}
		}
			
	}
	return $nb;	
}	
?>